<?php
defined( 'ABSPATH' ) or exit;
use League\Csv\Writer;
use League\Csv\Reader;
if ( ! class_exists( 'WES_Admin_Ajax' ) ) {
    class WES_Admin_Ajax{
        public function __construct(){
            add_action('wp_ajax_wes_check_product_sku',[&$this,'ajax_check_product_sku']);
            add_action('wp_ajax_wes_check_variation_sku',[&$this,'ajax_check_variation_sku']);
            add_action('wp_ajax_wes_validate_all_data',[&$this,'ajax_validate_all_data']);
            add_action('wp_ajax_wes_rename_variation',[&$this,'ajax_rename_variation']);
            add_action('wp_ajax_wes_rename_variable_products',[&$this,'ajax_rename_variable_products']);
            add_action('wp_ajax_wes_generate_sku',[&$this,'ajax_generate_sku']);
            add_action('wp_ajax_wes_check_duplicate_sku',[&$this,'ajax_check_duplicate_sku']);
            add_action('wp_ajax_wes_initiate_export',[&$this,'ajax_initiate_export']);
            add_action('wp_ajax_wes_export_product',[&$this,'ajax_export_product']);
        }
        private static function calculatePercentage($num_amount, $num_total){
            $count1 = $num_amount / $num_total;
            $count2 = $count1 * 100;
            $count = number_format($count2, 0);
            return $count;
        }
        public static function ajax_export_product(){
            check_ajax_referer( 'wes-export-product-nonce', 'security' );
           
            $productExportOffset=get_option('wes_product_export_offset',0);
            $product_args = [
                'posts_per_page' => Woo_Exporter_Sku::$perPageLimit,
                'post_status' => ['publish'],
                'post_type' => ['product','product_variation'],
                'order' => 'ASC',
                'offset'=>$productExportOffset
                ];
               $product_query=new WP_Query($product_args);
               
               if($product_query->found_posts > 0){
                $filePath=get_option('wes_exporter_file_path');
                $attrColum=get_option('wes_attrbiute_columns',[]);
                $records=self::csvArray($product_query->posts,$attrColum);
                $productExportOffset+=Woo_Exporter_Sku::$perPageLimit;
                update_option('wes_product_export_offset',$productExportOffset);
                $writer = Writer::createFromPath($filePath, 'a+');
                $writer->insertAll($records);
                $totalProducts=get_option('wes_total_products');
                if($totalProducts > 0){
                    $percentage=self::calculatePercentage($productExportOffset,$totalProducts);
                    wp_send_json_success(['percentage'=>$percentage > 100 ? 100 : $percentage]);

                }
               }else{
                   $totalProducts=get_option('wes_total_products');
                   ob_start();
                   ?>
                     <div class="wesNotificationIcon">
                <span class="dashicons dashicons-yes-alt green-icon"></span>
                </div>
                <h2>Export gereed voor download</h2>
                <p>
                 We exporteerden <?=$totalProducts?> producten uit jouw WooCommerce en je kunt deze nu downloaden als .csv bestand.
                </p>
             
                <div class="wesActionButtonContaier">
                    <button  data-target="<?php esc_attr_e(admin_url( 'admin.php?page=woo-download-csv' )) ?>" id="wesDownloadCsv"><span class="dashicons dashicons-download"></span> Download Export (.csv)</a>
                </div>
               
                   <?php

                   wp_send_json_success(['done'=>true,'html'=>ob_get_clean()]);
               }
               wp_send_json_success();
        }
        private static function csvArray($records, $attrColum){
                $returnResult=[];
                foreach($records as $k=>$post){
                    $productId=$post->ID;
                    $type=$post->post_type;
                    $product=  wc_get_product($productId);
                    if($product->get_type()=='variable')continue;
                    $sku=$product->get_sku();
                    $parentId=$post->post_parent;
                    $postType=$post->post_type;
                    $postTitle=$product->get_title();
                    $postDesc=$product->get_description();
                    $postShortDesc=$product->get_short_description();
                    $regularPrice=$product->get_regular_price();
                    $vatPercentage=0.0;
                    $tax_rates = WC_Tax::get_rates( $product->get_tax_class() );
                    if (!empty($tax_rates)) {
                        $tax_rate = reset($tax_rates);
                        $vatPercentage=$tax_rate['rate'];
                    }
                    $imageId=$product->get_image_id();
                    $gallery_attachment_ids =$product->get_gallery_image_ids();
                    if(empty($gallery_attachment_ids))[];
                    array_push($gallery_attachment_ids,$imageId);
                    $images='';
                    if(!empty($gallery_attachment_ids)){
                        $imageUrls=[];
                        foreach($gallery_attachment_ids as $attachmentimageId){
                            array_push($imageUrls,wp_get_attachment_url($attachmentimageId));
                        }
                        $images=implode('|',$imageUrls);
                    }
                    $terms = wc_get_product_terms(
                        $postType=='product_variation' && $parentId > 0 ? $parentId : $productId,
                        'product_cat',
                       
                            array(
                                'orderby' => 'parent',
                                'order'   => 'DESC',
                            )
                        
                    );
                    // $categories=[];
                    // if ( $terms ) {
                    //     $main_term =  $terms[0];
                    //     $term_ancestors=self::term_ancestors($main_term->term_id, 'product_cat');
                    //     $categories=$term_ancestors;
                    //     array_push($categories,html_entity_decode($main_term->name));
                    // }
                    // $wooCategories=implode(" - ",$categories);
                 
                    if(!empty($terms)){
                        $cats1=[];
                        $finalCats=[];
                        foreach($terms as $term){
                            $categories=[];
                            if(in_array($term->term_id,$cats1)) continue;
                            $term_ancestorIds=self::term_ancestorsId($term->term_id, 'product_cat');
                       
                          if(!empty($term_ancestorIds)){
                            $cats1=$term_ancestorIds;
                          }
                          array_push($cats1,$term->terrm_id);

                            $term_ancestors=self::term_ancestors($term->term_id, 'product_cat');
                            $categories=$term_ancestors;
                            array_push($categories,html_entity_decode($term->name));
                            $wooCategories1=implode(" - ",$categories);
                            array_push($finalCats,$wooCategories1);
                        }
                      $wooCategories=implode(', ',$finalCats);
                    }

               
                    $recordArr=[
                        $productId,
                        $type,
                        $sku,
                        $parentId,
                        $postTitle,
                        $postDesc,
                        $postShortDesc,
                        str_replace(".", ",", $regularPrice),
                        $vatPercentage,
                        $images,
                        $wooCategories,
                        $product->get_stock_quantity()
                    ];

                    
                    if(!empty($attrColum)){
                       

                        foreach($attrColum as $col){
                            $colValue=$product->get_attribute($col);
                            if(empty($colValue)){
                                $colValue=$product->get_attribute('pa_'.$col);
                            }
                           array_push($recordArr,$colValue);
                        }
                    }
                    $returnResult[$k]=$recordArr;
                }
                return $returnResult;

        }
        private static function term_ancestors( $term_id, $taxonomy) {
            $ancestors = get_ancestors( $term_id, $taxonomy );
            $ancestors = array_reverse( $ancestors );
            $ancestorArray=[];
            foreach ( $ancestors as $ancestor ) {
                $ancestor = get_term( $ancestor, $taxonomy );
                
                if ( ! is_wp_error( $ancestor ) && $ancestor ) {
                 
                   array_push($ancestorArray,$ancestor->name);
                }
            }
            return $ancestorArray;
        }
        private static function term_ancestorsId( $term_id, $taxonomy) {
            $ancestors = get_ancestors( $term_id, $taxonomy );
            $ancestors = array_reverse( $ancestors );
            $ancestorArray=[];
            foreach ( $ancestors as $ancestor ) {
                $ancestor = get_term( $ancestor, $taxonomy );
                
                if ( ! is_wp_error( $ancestor ) && $ancestor ) {
                 
                   array_push($ancestorArray,$ancestor->term_id);
                }
            }
            return $ancestorArray;
        }
        public static function ajax_initiate_export(){
            check_ajax_referer( 'wes-initiate-export-nonce', 'security' );
            $cusrrentTime=current_time('timestamp');
            $upload_dir      = wp_upload_dir();
            $wes_upload_dir=$upload_dir["basedir"]."/wes_store_csv";
            $filePath=$wes_upload_dir.'/'.$cusrrentTime.'.csv';

            $headerCSV=[
                'Woo ID',
                'Type',
                'SKU',
                'Product nummer',
                'Naam',
                'Omschrijving',
                'Korte omschrijving',
                'Prijs',
                'BTW tarief',
                'Afbeeldingen',
                'Categorieën',
                'Stock'
            ];
            $attributes_tax_slugs = array_keys( wc_get_attribute_taxonomy_labels() );
            if(!empty($attributes_tax_slugs)){
                update_option('wes_attrbiute_columns',$attributes_tax_slugs);
                $headerCSV=array_merge($headerCSV,$attributes_tax_slugs);
            }
           $writer = Writer::createFromPath($filePath, 'w+');
           $writer->insertOne($headerCSV);
            update_option('wes_exporter_file_path',$filePath);
            $product_args = [
                'posts_per_page' => -1,
                'post_status' => ['publish'],
                'post_type' => ['product','product_variation'],
                ];
                $productQuery=new WP_Query( $product_args );
                $totalProducts=$productQuery->found_posts;
                update_option('wes_total_products',$totalProducts);
                wp_send_json_success(['totalData'=>$totalProducts]);
        }
        public static function ajax_check_duplicate_sku(){
            check_ajax_referer( 'wes-duplicate-check-nonce', 'security' );
            $duplicateSkuIds=get_option('wes_duplicate_product_skus',[]);
            if(!empty($duplicateSkuIds)){
                foreach($duplicateSkuIds as $productId){
                    $product=wc_get_product($productId);
                    $sku=$product->get_sku();
                    $checkSku=self::getDuplicateId($sku,$productId);
                    if(!$checkSku){
                        if (($key = array_search($productId, $duplicateSkuIds)) !== false) {
                            unset($duplicateSkuIds[$key]);
                        }
                        
                    }
                }
                update_option('wes_duplicate_product_skus',$duplicateSkuIds);
            }
            wp_send_json_success();
        }
        public static function ajax_generate_sku(){
            check_ajax_referer( 'wes-generate-sku-nonce', 'security' );
            $emptyProductIDs=get_option('wes_empty_product_skus');
            if(!empty($emptyProductIDs)){
                foreach($emptyProductIDs as $productId){
                    update_post_meta($productId,'_sku',$productId);
                }
                delete_option('wes_empty_product_skus');
            }
            wp_send_json_success();
        }
        public static function ajax_rename_variation(){
            check_ajax_referer( 'wes-rename-variation-nonce', 'security' );
            $variationIds=get_option('wes_variation_skus');
          //  $alreadyAddedintoDB=get_option('wes_variation_sku_added_to_db',[]);
            if(!empty($variationIds)){
                foreach($variationIds as $variationId){
                    update_post_meta($variationId,'_sku',$variationId);
                  //  array_push($alreadyAddedintoDB,$variationId);
                    
                }
             //   update_option('wes_variation_sku_added_to_db',$alreadyAddedintoDB);
                delete_option('wes_variation_skus');
            }
            wp_send_json_success();
        }
        public static function ajax_rename_variable_products(){
            check_ajax_referer( 'wes-rename-variaable-products-nonce', 'security' );
             $variableProductIds=get_option('wes_found_variable_products');
             $alreadyGivenVariableIds=get_option('wes_done_variable_products',[]);
             if(!empty($variableProductIds)){
                foreach($variableProductIds as $variableProductId){
                    update_post_meta($variableProductId,'_sku','variation-'.$variableProductId);
                    array_push($alreadyGivenVariableIds,$variableProductId);

                }
                update_option('wes_done_variable_products',$alreadyGivenVariableIds);
                delete_option('wes_found_variable_products');
             }
             wp_send_json_success();
        }
        public static function ajax_validate_all_data(){
            check_ajax_referer( 'wes-validate-all-data-nonce', 'security' );
            $isOk=true;
            $emptyProductIDs=get_option('wes_empty_product_skus');
            if(!empty($emptyProductIDs)){
                ?>
                 <div class="wesNotificationIcon">
                <span class="dashicons dashicons-info red-icon"></span>
                </div>
                <h2>We vonden <?=count($emptyProductIDs)?> producten zonder bruikbare SKU/Artikelnummer.</h2>
                <p>
                Alle producten moeten een unieke en nummerieke SKU/Artikelnummer hebben. 
                </p>
                <p>
                Wil je alle producten automatisch een geldige en unieke SKU/Artikelnummer geven? Bestaande waardes kunnen hierdoor overschreven worden.
                </p>
                <div class="wesActionButtonContaier">
                    <button id="wesGenerateSKU">Ja, geef alle prolducten een unieke SKU</button>
                </div>

                <?php
                $isOk=false;
                     wp_send_json_success(['html'=>ob_get_clean(),'isOk'=>$isOk]);
            }
            $duplicateProductSku=get_option('wes_duplicate_product_skus');
            if(!empty($duplicateProductSku)){
                ?>
                 <div class="wesNotificationIcon">
                <span class="dashicons dashicons-info red-icon"></span>
                </div>
                <h2>We vonden <?=count($duplicateProductSku)?> producten die geen unieke SKU/Artikelnummer hebben</h2>
                <p>Een bruikbare export vereist unieke SKU/Artikelnummers.</p>
                <p>Update aub de SKU/Artikelnummer van de volgende artikelen zodat deze uniek worden.</p>
                <p>
                  
                </p>
                <?php
                  $isOk=false;
                  wp_send_json_success(['html'=>ob_get_clean(),'isOk'=>$isOk,'tableData'=>self::getDuplicateSkuHTML($duplicateProductSku),'showContinue'=>true]);
            }
            $variableProductIds=get_option('wes_found_variable_products');
            if(!empty($variableProductIds)){
                ob_start();
                ?>
                 <div class="wesNotificationIcon">
                <span class="dashicons dashicons-info red-icon"></span>
                </div>
                <h2>We vonden <?=count($variableProductIds)?> variabele producten.</h2>
                <p>
                Voor een bruikbare export is het noodzakelijk dat alle variaties een unieke en nummerieke SKU hebben.
                </p>
                <p>
                Voor variabele producten geldt dat we de SKU/Artikelnummer aanpassen naar 'variation-' gevolgd door de Post ID. 
                </p>
                <div class="wesActionButtonContaier">
                    <button id="wesRenameVariableSku">Controleer en hernoem alle SKU/Artikelnummers</button>
                </div>
                <p>
                <span class="dashicons dashicons-info yellow-icon"></span> <strong>WAARSCHUWING:</strong> Deze actie kan niet ongedaan worden gemaakt!
                </p>
                <?php
                  $isOk=false;
                  wp_send_json_success(['html'=>ob_get_clean(),'isOk'=>$isOk]);
            }
            $variationIds=get_option('wes_variation_skus');

            if(!empty($variationIds)){
                ob_start();
                ?>
                <div class="wesNotificationIcon">
                <span class="dashicons dashicons-info red-icon"></span>
                </div>
                <h2>We vonden <?=count($variationIds)?> variaties.</h2>
                <p>
                Een bruikbare export vereist unieke SKU/Artikelnummers voor alle variaties.
                </p>
                <p>
                Voor variaties passen wij de SKU/Artikelnummer aan naar de Post ID zodat deze een unieke en numerieke waarde krijgen.
                </p>
                <div class="wesActionButtonContaier">
                <button id="wesRenameVariationSku">Controleer en hernoem alle SKU/Artikelnummers</button>
                </div>
                <p>
                <span class="dashicons dashicons-info yellow-icon"></span> <strong>WAARSCHUWING:</strong> Deze actie kan niet ongedaan worden gemaakt!
                </p>
                <?php
                $isOk=false;
                wp_send_json_success(['html'=>ob_get_clean(),'isOk'=>$isOk]);
            }
            ?>
              <div class="wesNotificationIcon">
              <span class="dashicons dashicons-yes-alt green-icon"></span>
                </div>
                <h2>Alle controles zijn voltooid</h2>
                <h2>Je bent nu klaar om alle producten te exporteren.</h2>
                <p>De duur van de export hangt af van het aantal producten die in je WooCommerce staan.</p>
            <?php
            wp_send_json_success(['html'=>ob_get_clean(),'isOk'=>$isOk]);
        }
        private static function getDuplicateSkuHTML($productIds){
         
            ?>
            <table class="pure-table pure-table-horizontal widefat wesTable">
            <?php
            foreach($productIds as $productId){
                $product=wc_get_product($productId);
                $sku=$product->get_sku();
                ?>
                <tr>
                    <td>
                        #<?=$productId?>
                    </td>
                    <td><a target="_blank" href="<?=get_edit_post_link($productId)?>"><?=$product->get_title()?></a></td>
                    <td><?=$sku?></td>
                    <td>Heeft dezelfde SKU/Artikelnummer als 
                        <?php
                            $duplicateProduct=self::getDuplicateId($sku,$productId);
                            if($duplicateProduct){
                                $duplicateProductId=$duplicateProduct->ID;
                               ?>
                               <a target="_blank" href="<?=get_edit_post_link($duplicateProductId)?>">#<?=$duplicateProductId?></a>
                               <?php
                            }
                         ?>
                    </td>
                    <td><a target="_blank" href="<?=get_edit_post_link($productId)?>">SKU/Artikelnummer bijwerken</a></td>
                </tr>
           <?php }
             ?>
            </table>
            <?php
            return ob_get_clean();
        }
        private static function getDuplicateId($sku,$postId){
            $product_args = [
                'posts_per_page' => -1,
                'post_status' => ['publish'],
                'post_type' => ['product'],
                'meta_key'=>'_sku',
                'meta_value'=>$sku,
                'post__not_in'=>[$postId]
                ];
                $skuQuery=new WP_Query( $product_args );
                if($skuQuery->found_posts > 1){
                    return $skuQuery->posts[0];
                }
                return false;
        }
        public static function ajax_check_variation_sku(){
            check_ajax_referer( 'wes-check-variation-sku-nonce', 'security' );
            $variationSkuOffset=get_option('wes_variation_offset',0);
            $existingVariationSku=get_option('wes_variation_skus',array());
            // $alreadyAddedintoDB=get_option('wes_variation_sku_added_to_db',[]);
            $product_args = [
                'posts_per_page' => Woo_Exporter_Sku::$perPageLimit,
                'post_status' => ['publish'],
                'post_type' => ['product_variation'],
                'order' => 'ASC',
                'offset'=>$variationSkuOffset
                ];
                $vairationQuery = new WP_Query($product_args);
                $product_args = [
                    'posts_per_page' => -1,
                    'post_status' => ['publish'],
                    'post_type' => ['product_variation'],
                    ];
                    $productQuery=new WP_Query( $product_args );
                    $totalProducts=$productQuery->found_posts;
                    $limitedProducts= $variationSkuOffset + Woo_Exporter_Sku::$perPageLimit;
                    $to= $limitedProducts >= $totalProducts ? $totalProducts : $limitedProducts;
                    $from=$variationSkuOffset + 1;
                if(!empty($vairationQuery->posts)){
                    foreach($vairationQuery->posts as $variation){
                        $isEligable=false;
                        $variationId=$variation->ID;
                        $variationSku=get_post_meta($variationId,'_sku',true);
                        if(!empty($variationSku)){
                            $checkSku=self::checkDuplicateSKU($variationSku);
                            if($checkSku){
                                $isEligable=true;
                            }
                            if(!is_int($variationSku)){
                                $isEligable=true;
                            }
                        }else{
                            $isEligable=true;
                        }
                        if($isEligable && !in_array($variationId,$existingVariationSku)){
                            array_push($existingVariationSku,$variationId);
                        }
                        $variationSkuOffset++;
                    }
                    update_option('wes_variation_skus',$existingVariationSku);
                    update_option('wes_variation_offset',$variationSkuOffset);
                }else{
                    delete_option('wes_variation_offset');
                    wp_send_json_success(['done'=>true]);
                }
                wp_send_json_success(['from'=>$from,'to'=>$to,'total'=>$totalProducts]);
        }
        private static function checkDuplicateSKU($sku){
            $product_args = [
                'posts_per_page' => -1,
                'post_status' => ['publish'],
                'post_type' => ['product'],
                'meta_key'=>'_sku',
                'meta_value'=>$sku
                ];
                $skuQuery=new WP_Query( $product_args );
               
                return $skuQuery->found_posts > 1 ? true : false;
        }
        public static function ajax_check_product_sku(){
            check_ajax_referer( 'wes-check-product-sku-nonce', 'security' );
            $productSkuOffset=get_option('wes_product_offset',0);
            $emptyProductSku=get_option('wes_empty_product_skus',[]);
            $duplicateProductSkuIds=get_option('wes_duplicate_product_skus',[]);
            $variableProductIds=get_option('wes_found_variable_products',[]);
            $alreadyGivenVariableIds=get_option('wes_done_variable_products',[]);

            if(empty($emptyProductSku))$emptyProductSku=[];
            $args=[
                'status' => 'publish',
                'paginate' => true,
                'limit' => Woo_Exporter_Sku::$perPageLimit,
                'offset' => $productSkuOffset,
              
            ];
            $products=wc_get_products($args);
         
            $product_args = [
                'posts_per_page' => -1,
                'post_status' => ['publish'],
                'post_type' => ['product'],
                ];
                $productQuery=new WP_Query( $product_args );
                $totalProducts=$productQuery->found_posts;
                $limitedProducts= $productSkuOffset + Woo_Exporter_Sku::$perPageLimit;
                $to= $limitedProducts >= $totalProducts ? $totalProducts : $limitedProducts;
                $from=$productSkuOffset + 1;

            if(!empty($products->products)){
                foreach($products->products as $product){
                    $sku=$product->get_sku();
                    $productId=$product->get_id();
                    $productSkuOffset+=1;
                    if($product->get_type()=='variable'){
                        // if(!in_array($productId,$alreadyGivenVariableIds)){
                        //     array_push($variableProductIds,$productId);
                        // }
                        if(empty($sku)){  
                            if(!in_array($productId,$alreadyGivenVariableIds)){
                                array_push($variableProductIds,$productId);
                            }
                        }
                        if(!empty($sku)){
                            $checkSku=self::checkDuplicateSKU($sku);
                            if($checkSku){
                                if(!in_array($productId,$alreadyGivenVariableIds)){
                                    array_push($variableProductIds,$productId);
                                }
                            }
                            if(!is_int($sku)){
                                if(!in_array($productId,$alreadyGivenVariableIds)){
                                    array_push($variableProductIds,$productId);
                                } 
                            }
                        }

                        continue;
                    }
                    if(empty($sku)){  
                        if(is_array($emptyProductSku) && !in_array($productId,$emptyProductSku)){
                            array_push($emptyProductSku,$productId);
                        }
                    }
                    if(!empty($sku)){
                        $checkSku=self::checkDuplicateSKU($sku);
                        if($checkSku){
                            if(!in_array($productId,$duplicateProductSkuIds)){
                                array_push($duplicateProductSkuIds,$productId);
                            }
                        }
                        if(!is_int($sku)){
                            if(is_array($emptyProductSku) && !in_array($productId,$emptyProductSku)){
                                array_push($emptyProductSku,$productId);
                            }
                        }
                    }
                  
                  
                }
                if(!empty($emptyProductSku)){
                    update_option('wes_empty_product_skus',$emptyProductSku);
                }
                if(!empty($duplicateProductSkuIds)){
                    update_option('wes_duplicate_product_skus',$duplicateProductSkuIds);
                }
                if(!empty($variableProductIds)){
                    update_option('wes_found_variable_products',$variableProductIds);
                }
             
                update_option('wes_product_offset',$productSkuOffset);
               
            }else{
             
                delete_option('wes_product_offset');
                wp_send_json_success(['done'=>true]);
            }
            wp_send_json_success(['from'=>$from,'to'=>$to,'total'=>$totalProducts]);
        }
    }
    new WES_Admin_Ajax();
}