<?php
/**
 * Plugin Name: Mplus WooCommerce Exporter
 * Description: Export Woocommerce products matched with SKU
 * Version: 1.5
 * Text Domain: woo-exporter-sku
 * Domain Path: /languages
 * Author: MplusWEBSHOPS
 * Author URI: https://mpluswebshops.nl/
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */


if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
if (!defined('WES_PLUGIN_DIR'))
    define( 'WES_PLUGIN_DIR', dirname(__FILE__) );
if (!defined('WES_PLUGIN_ROOT_PHP'))
    define( 'WES_PLUGIN_ROOT_PHP', dirname(__FILE__).'/'.basename(__FILE__)  );
if(!defined('WES_PLUGIN_ABSOLUTE_PATH'))
    define('WES_PLUGIN_ABSOLUTE_PATH',plugin_dir_url(__FILE__));
if (!defined('WES_PLUGIN_ADMIN_DIR'))
    define( 'WES_PLUGIN_ADMIN_DIR', dirname(__FILE__) . '/admin' );
if (!defined('WES_TEXT_DOMAIN'))
    define( 'WES_TEXT_DOMAIN', 'woo-exporter-sku' );
if( !class_exists('Woo_Exporter_Sku') ) {
    class Woo_Exporter_Sku{
        public static  $perPageLimit=100;
        public function __construct()
        {
            if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
                require_once(WES_PLUGIN_DIR.'/vendor/autoload.php');
                require_once(WES_PLUGIN_ADMIN_DIR.'/class-admin.php');
                add_filter('plugin_action_links_'.plugin_basename(__FILE__), [&$this,'add_setting_link']);
                add_action( 'activated_plugin', [&$this,'activation_redirect'] );

            }else{
                add_action( 'admin_notices',[&$this,'add_notice_for_woocommerce']);
            }
        }
        public function activation_redirect( $plugin){
            if( $plugin == plugin_basename( __FILE__ ) ) {
                exit( wp_redirect( admin_url( 'admin.php?page=woo-export' ) ) );
            }
        
        }
        public function add_setting_link($links){
            array_unshift($links, '<a href="' .
                admin_url( 'admin.php?page=woo-export' ) .
                '">' . esc_attr__('Configure',WES_TEXT_DOMAIN) . '</a>');
            return $links;
        }
        public static function add_notice_for_woocommerce(){
            ?>
            <div class="notice notice-error">
                <p><strong><?php esc_attr_e( 'Woo Exporter requires the WooCommerce plugin to be installed and active.', WES_TEXT_DOMAIN); ?></strong></p>
            </div>
            <?php
        }
        public static function install(){
            $upload_dir      = wp_upload_dir();
            $wes_upload_dir=$upload_dir["basedir"]."/wes_store_csv";
            $content=file_get_contents(plugin_dir_path(__FILE__).'/htaccess.txt');
            $files = [
                [
                    'base' 		=> $wes_upload_dir,
                    'file' 		=> 'index.html',
                    'content' 	=> 'Silent is bliss'
                ],
                [
                    'base' 		=> $wes_upload_dir,
                    'file' 		=> '.htaccess',
                    'content' 	=> $content
                ]
            ];
            foreach ( $files as $file ) {
                if ( wp_mkdir_p( $file['base'] ) && ! file_exists( trailingslashit( $file['base'] ) . $file['file'] ) ) {
                    if ( $file_handle = @fopen( trailingslashit( $file['base'] ) . $file['file'], 'w' ) ) {
                        fwrite( $file_handle, $file['content'] );
                        fclose( $file_handle );
                    }
                }
            }
          //  exit(wp_safe_redirect(  admin_url( 'admin.php?page=woo-export' )));
        }
       

    }
    register_activation_hook( __FILE__, ['Woo_Exporter_Sku', 'install' ]);
    new Woo_Exporter_Sku();
}